#include <stdio.h>
#if defined WIN32
#define WINDOWS_LEAN_AND_MEAN
#include <windows.h>
#else
#include <limits.h>
#include <unistd.h>
#include <sys/times.h>
#endif

#if defined AMXMODX
#include "amxxmodule.h"
#elif defined AMXMOD
#include "modules.h"

#ifndef AMX_IFACE_VERSION
#define AMX_IFACE_VERSION 9
#endif

module_info_s module_info = {
	"Bench",
	"AMXx Dev Team",
	"1.00",
	AMX_IFACE_VERSION,
	RELOAD_MODULE,
};

pfnamx_engine_g* g_engAmxFunc;
pfnmodule_engine_g* g_engModuleFunc;
#endif

extern AMX_NATIVE_INFO g_natives[];

static cell AMX_NATIVE_CALL get_tick_count(AMX *amx, cell *params)
{
#if defined WIN32
	return static_cast<cell>(GetTickCount());
#else
	tms tm;
	clock_t ticks = times(&tm);
	long ticks_per_sec = sysconf(_SC_CLK_TCK);
	/* do some annoying math */
	double fticks = (double)ticks / (double)ticks_per_sec;
	fticks *= 1000.0f;
	// reduce to int size - this should usually be okay.
	if (fticks > INT_MAX)
	{	
		double r = (int)(fticks / INT_MAX) * (double)INT_MAX;
		fticks -= r;
	}
	return static_cast<cell>(fticks);
#endif
}

#if defined AMXMODX
void OnAmxxAttach()
{
	MF_AddNatives(g_natives);
}
#elif defined AMXMOD
C_DLLEXPORT int AMX_Query(module_info_s** info)
{
	//HACKHACK: the version numbers aren't backwards compatible
	// nor are they public (AMX2005 sources aren't available)
	// and the version in CVS doesn't match what's compiled.
	//you can find the numbers through disassembly, however,
	// we are going to patch it at run time and ignore the fact
	// that the function tables will be different.
	//I'm doing this by detecting a smaller amx_mm.dll
	// == amx2k5, since I don't have time for anything more clever.
	//this only works on cstrike, mod path isn't available
	// since we don't want to attach to MM.
#if defined WIN32
	const char *file = "cstrike\\addons\\amx\\dlls\\amx_mm.dll";
#else
	const char *file = "cstrike/addons/amx/dlls/amx_mm.dll";
#endif
	FILE *fp = fopen(file, "rb");
	if (fp)
	{
		fseek(fp, 0, SEEK_END);
		long offs = ftell(fp);
		if (offs > 160000)
			module_info.ivers = 8;
		fclose(fp);
	}
	*info = &module_info;
	return 1;
}

C_DLLEXPORT int AMX_Attach(pfnamx_engine_g* aeng,pfnmodule_engine_g* meng)
{
	g_engAmxFunc = aeng;
	g_engModuleFunc = meng;

	ADD_AMXNATIVES(&module_info, g_natives);

	return 1;
}

C_DLLEXPORT int AMX_Detach()
{
	return 1;
}
#endif

AMX_NATIVE_INFO g_natives[] = 
{
	{"get_tick_count",		get_tick_count},
	{NULL,					NULL},
};
